package com.zy.bms.service;

import com.alibaba.fastjson.JSON;
import com.zy.bms.common.Constant;
import com.zy.bms.config.mqtt.MqttGateway;
import com.zy.bms.model.Device;
import com.zy.bms.model.MqttLog;
import com.zy.bms.model.MqttMsgDto;
import com.zy.bms.common.io.wechat.*;
import com.zy.bms.utils.HttpUtil;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Component;
import org.springframework.util.MultiValueMap;

import javax.annotation.Resource;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;

/**
 * 设置设备服务类,发送远程消息
 *
 * @author yang xiao kun
 * create on 2021/2/1
 */
@Component
public class SetToDeviceService {
    @Resource
    private MqttGateway mqttGateway;
    @Resource
    private MqttLogService mqttLogService;

    /**
     * 发送消息
     *
     * @param msg   消息
     * @param topic 主题
     */
    private void sendMsgToMqtt(String msg, String topic) {
        //存储记录
        MqttLog mqttLog = new MqttLog();
        mqttLog.setTopic(topic);
        mqttLog.setTag(1);
        //设备注册报文
        if (topic.equals(Constant.TOPIC_REGISTER_CLIENT)) {
            MqttMsgDto msgObject = JSON.parseObject(msg, MqttMsgDto.class);
            mqttLog.setDeviceId(msgObject.getString("regnum"));
        } else {
            mqttLog.setDeviceId(topic.split("/")[1]);
        }
        mqttLog.setContent(msg);
        mqttLogService.save(mqttLog);
        //发送消息
        mqttGateway.sendMsgToMqtt(msg, topic);
    }

    /**
     * 设置音量
     * 铃声音量暂时不可用 ringvol 参数暂时没用
     *
     * @param io 设置参数
     */
    public void setVolume(VolumeSetIO io) {
        //发送模板消息给设备
        MqttMsgDto msg = new MqttMsgDto(Constant.M_CODE_VOLUME, io.getDeviceId());
        msg.set("msgvol", io.getMsgVol())
                .set("phonevol", io.getPhoneVol())
                .set("msgtype", 1);
        sendMsgToMqtt(msg.toJson(), io.getTopic());
    }

    /**
     * 设置定位频率
     *
     * @param io 设置参数
     */
    public void setGpsRate(GpsRateSetIO io) {
        //发送模板消息给设备
        MqttMsgDto msg = new MqttMsgDto(Constant.M_CODE_GPS_RATE, io.getDeviceId());
        msg.set("GPS_rate", io.getGpsRate());
        sendMsgToMqtt(msg.toJson(), io.getTopic());
    }

    /**
     * 自动接听
     *
     * @param io 设置参数
     */
    public void setAutoAnswer(AnswerSetIO io) {
        //发送模板消息给设备
        MqttMsgDto msg = new MqttMsgDto(Constant.M_CODE_AUTO_ANSWER, io.getDeviceId());
        msg.set("autoanswer", io.getAutoAnswer());
        sendMsgToMqtt(msg.toJson(), io.getTopic());
    }

    /**
     * 设置语音播报
     */
    public void sendNews(NewsSetIO io) {
        //发送模板消息给设备
        MqttMsgDto msg = new MqttMsgDto(Constant.M_CODE_NEWS, io.getDeviceId());
        msg.set("newstype", io.getNewsType())
                .set("newstime", io.getNewsTime())
                .set("news", io.getNews());
        sendMsgToMqtt(msg.toJson(), io.getTopic());
    }

    /**
     * 设置SOS紧急呼叫
     */
    public void setSOS(SosSetIO io) {
        //发送模板消息给设备
        MqttMsgDto msg = new MqttMsgDto(Constant.M_CODE_SOS, io.getDeviceId());
        msg.set("key0_name", io.getKey0Name())
                .set("key0_phone", io.getKey0Phone())
                .set("key1_name", io.getKey1Name())
                .set("key1_phone", io.getKey1Phone())
                .set("key2_name", io.getKey2Name())
                .set("key2_phone", io.getKey2Phone());
        sendMsgToMqtt(msg.toJson(), io.getTopic());
    }

    /**
     * 设置持续定位
     */
    public void setContinue(ContinueSetIO io) {
        //发送模板消息给设备
        MqttMsgDto msg = new MqttMsgDto(Constant.M_CODE_CONTINUE_GPS, io.getDeviceId());
        msg.set("highfreq", io.getHighFreq());
        sendMsgToMqtt(msg.toJson(), io.getTopic());
    }

    /**
     * 服务器返回授权码
     */
    public void returnCodeToDevice(Device device) {
        //发送模板消息给设备
        MqttMsgDto msg = new MqttMsgDto();
        msg.setM(Constant.M_CODE_RETURN_CODE);
        msg.setR(Constant.TOPIC_REGISTER_CLIENT);
        msg.setT(Constant.TOPIC_REGISTER_SERVER);
        Map<String, Object> data = new HashMap<>();
        data.put("regnum", device.getNum());
        data.put("deviceid", device.getClientId());
        data.put("password", device.getPassword());
        msg.setData(data);
        registerMqtt(device.getClientId(),device.getPassword());
        sendMsgToMqtt(msg.toJson(), Constant.TOPIC_REGISTER_CLIENT);
    }

    /**
     * 向 MQTT 服务器申请用户,供设备进行登录
     * <p>
     * 因为存在先后问题,需要先注册用户,在开通虚拟机权限,
     * 所以逻辑为
     * 发送注册用户请求
     * 尝试申请权限
     * 如果失败,则间隔一秒再次请求
     * 如果申请次数超过3次,则尝试重新注册
     * 重新注册后再次尝试申请
     * 如果申请五次都失败,则结束方法,记录日志
     *
     * @param clientId 设备登录ID
     * @param password 设备登录密码
     */
    public void registerMqtt(String clientId, String password) {
        addRabbitMqUser(clientId, password);
        int count = 0;
        while (!setPermission(clientId)) {
            try {
                Thread.sleep(1000);
                if (count == 2) {
                    addRabbitMqUser(clientId, password);
                }
                if (count > 4) {
                    break;
                }
                count++;
            } catch (InterruptedException e) {
                e.printStackTrace();
                break;
            }
        }
    }

    /**
     * 注册 RabbitMQ 用户
     *
     * @param clientId 用户名
     * @param password 用户密码
     */
    public void addRabbitMqUser(String clientId, String password) {
        String url = "http://view.ailishi.org:15672/api/users/" + clientId;
        //请求头
        HttpHeaders headers = new HttpHeaders();
        headers.add("authorization", "Basic bHEyMDE5OkxpUXVhblJhYmJpdA==");
        //请求参数
        Map<String, String> params = new HashMap<>();
        params.put("username", clientId);
        params.put("password", password);
        params.put("tags", "");
        HttpUtil.putForJSONEntity(url, headers, JSON.toJSONString(params));
    }

    /**
     * 申请虚拟机权限
     *
     * @param clientId 用户ID
     */
    public boolean setPermission(String clientId) {
        String url = "http://view.ailishi.org:15672/api/permissions/%2F/" + clientId;
        //请求头
        HttpHeaders headers = new HttpHeaders();
        headers.add("authorization", "Basic bHEyMDE5OkxpUXVhblJhYmJpdA==");
        //请求参数
        Map<String, String> params = new HashMap<>();
        params.put("username", clientId);
        params.put("vhost", "/");
        params.put("configure", ".*");
        params.put("write", ".*");
        params.put("read", ".*");
        try {
            HttpUtil.putForJSONEntity(url, headers, JSON.toJSONString(params));
        } catch (Exception e) {
            return false;
        }
        return true;
    }
}
